package cz.muni.fi.pv168.project.ui.component;

import javax.swing.DefaultListModel;
import javax.swing.JList;
import javax.swing.ListCellRenderer;
import javax.swing.border.TitledBorder;
import java.util.ArrayList;
import java.util.Collections;
import java.util.List;
import java.util.Optional;

public final class ModeledJList<T> extends JList<T> {

    private DefaultListModel<T> model;

    public ModeledJList(ListCellRenderer<T> renderer) {
        this(Optional.empty(), renderer, new ArrayList<>());
    }

    public ModeledJList(String listName, ListCellRenderer<T> renderer, List<T> data) {
        this(Optional.of(listName), renderer, data);
    }

    private ModeledJList(Optional<String> borderTitle, ListCellRenderer<T> renderer, List<T> data) {
        model = new DefaultListModel<>();
        model.addAll(data);
        this.setModel(model);

        this.setCellRenderer(renderer);
        this.setEnabled(true);
        this.setOpaque(false);

        borderTitle.ifPresent(name -> this.setBorder(new TitledBorder(name)));
    }

    public void setCustomListModel(DefaultListModel<T> customListModel) {
        super.setModel(customListModel);
        this.model = customListModel;
    }

    public List<T> getValues() {
        return Collections.list(model.elements());
    }

    public void setValues(List<T> values) {
        model.clear();
        model.addAll(values);
    }
}
