package cz.muni.fi.pv168.project.ui.dialog.factory;

import cz.muni.fi.pv168.project.data.Identifiable;
import cz.muni.fi.pv168.project.data.task.Task;
import cz.muni.fi.pv168.project.db.interfaces.DataAccessObject;
import cz.muni.fi.pv168.project.db.interfaces.MtoNDao;
import cz.muni.fi.pv168.project.ui.dialog.AbstractLaidOutDialog;
import cz.muni.fi.pv168.project.ui.dialog.ComboBoxDialog;

import java.util.stream.Collectors;

public class DependencyTaskDialogFactory implements DialogFactory<Task> {

    private final DataAccessObject<Task> taskDao;
    private final MtoNDao dependencyDao;

    private Identifiable sourceEntity;

    public DependencyTaskDialogFactory(MtoNDao dependencyDao, DataAccessObject<Task> taskDao) {
        this.taskDao = taskDao;
        this.dependencyDao = dependencyDao;
    }

    @Override
    public AbstractLaidOutDialog<Task> newEditDialog(Task entity) {
        throw new UnsupportedOperationException();
    }

    @Override
    public AbstractLaidOutDialog<Task> newAddDialog() {
        var allowedTasks = taskDao
                .getAll()
                .stream()
                .filter(this::isLoopFree)
                .filter(this::isNewDependency)
                .collect(Collectors.toList());
        return new ComboBoxDialog<>(allowedTasks);
    }

    private boolean isNewDependency(Task otherTask) {
        for (Long targetId : dependencyDao.getAllIdsFor(sourceEntity.getId())) {
            if (otherTask.getId().equals(targetId)) {
                return false;
            }
        }

        return true;
    }

    private boolean isLoopFree(Task otherTask) {
        if (otherTask.getId().equals(sourceEntity.getId())) {
            return false;
        }

        for (Long targetId : dependencyDao.getAllIdsFor(otherTask.getId())) {
            if (targetId.equals(sourceEntity.getId())) {
                return false;
            }
        }

        return true;
    }

    public void setSourceEntity(Identifiable sourceEntity) {
        this.sourceEntity = sourceEntity;
    }
}
